/*
 * file NoTypeSorter.java
 * 
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM - you are allowed to copy, modify and 
 * redistribute this file as part of any program that interfaces with 
 * IBM Rational CM API.
 *
 * com.ibm.rational.teamapi.scout.NoTypeSorter
 *
 * (C) Copyright IBM Corporation 2004, 2008.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */
package com.ibm.rational.teamapi.scout;

import org.eclipse.jface.viewers.ContentViewer;
import org.eclipse.jface.viewers.IBaseLabelProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;

/**
 * A ViewSorter that sorts the bound member list on the simple name of the child
 * (as opposed to sorting on the label displayed in the tree view, which includes
 * resource type information).
 */
public class NoTypeSorter extends ViewerSorter {	
    /*
	 * @see ViewerSorter#compare(Viewer, Object, Object)
	 */
	public int compare(Viewer viewer, Object e1, Object e2) {
        int cat1 = category(e1);
        int cat2 = category(e2);
        
        if (cat1 != cat2) return cat1 - cat2;

        String name1, name2;
		if (viewer == null || !(viewer instanceof ContentViewer)) {
			name1 = e1.toString();
			name2 = e2.toString();
		} else {
			IBaseLabelProvider prov = ((ContentViewer)viewer).getLabelProvider();
			if (prov instanceof ILabelProvider) {
				ILabelProvider lprov = (ILabelProvider)prov;
				name1 = lprov.getText(e1);
				name2 = lprov.getText(e2);
			} else {
				name1 = e1.toString();
				name2 = e2.toString();
			}
		}
		return collator.compare(stripType(name1, e1), stripType(name2, e2));
	}

    /**
     * Strips the resource type from the head of the display name
     * @param name The element display name
     * @param obj The ProxyElement from which the name was derived
     * @return The element display name stripped of its resource type.
     */
	protected String stripType(String name, Object obj) {
        if (name == null)
            return "";
        
        if (obj!=null && (obj instanceof ProxyElement)) {
            int space = name.indexOf(" ");
        
            if (space > 0) {
                String type = name.substring(0,space);
                ProxyElement pe = (ProxyElement)obj;
                
                if (type.equals(pe.resourceType())) {
                    return name.substring(space+1);
                }
            }
        }
        
		return name;	
	}

}
